@@ -6,9 +6,16 @@ use glium::glutin::{
 use image::io::Reader as ImageReader;
 use imgui::*;
 use imgui_glium_renderer::Renderer;
-use std::{fs, io::Cursor, path::Path, process::Command, thread};
+use std::{fs, io::Cursor, path::Path, process::Command, thread, time::{Instant, Duration}};
 
-use super::{image_view::ImageView, vec2::Vec2, UserEvent};
+use super::{vec2::Vec2, UserEvent};
+
+mod image_view;
+use image_view::ImageView;
+mod image_list;
+use image_list::ImageList;
+mod arrows;
+use arrows::{Arrows, Action};
 
 macro_rules! min {
     ($x: expr) => ($x);
@@ -35,6 +42,8 @@ pub struct App {
     mouse_position: Vec2<f32>,
     current_filename: String,
     about_visible: bool,
+    image_list: ImageList,
+    arrows: Arrows,
 }
 
 impl App {
@@ -54,17 +63,30 @@ impl App {
 
         if let Some(event) = user_event {
             match event {
-                UserEvent::ImageLoaded(image, path) => {
-                    self.image_view = Some(ImageView::new(display, image.clone(), path.clone()));
-                    let view = self.image_view.as_mut().unwrap();
-                    self.current_filename = path.file_name().unwrap().to_str().unwrap().to_string();
+                UserEvent::ImageLoaded(image, path, instant) => {
+                    let replace = if let Some(ref old) = self.image_view {
+                        if old.start.saturating_duration_since(*instant) == Duration::from_secs(0) {
+                            true
+                        } else {
+                            false
+                        }
+                    } else {
+                        true
+                    };
 
-                    let scaling = min!(
-                        self.size.x() / view.size.x(),
-                        (self.size.y() - TOP_BAR_SIZE) / view.size.y()
-                    );
-                    view.scale = min!(scaling, 1.0);
-                    view.position = self.size / 2.0;
+                    if replace {
+                        self.image_view = Some(ImageView::new(display, image.clone(), path.clone(), *instant));
+                        let view = self.image_view.as_mut().unwrap();
+                        self.current_filename = path.file_name().unwrap().to_str().unwrap().to_string();
+                        self.image_list.change_dir(path);
+
+                        let scaling = min!(
+                            self.size.x() / view.size.x(),
+                            (self.size.y() - TOP_BAR_SIZE) / view.size.y()
+                        );
+                        view.scale = min!(scaling, 1.0);
+                        view.position = self.size / 2.0;
+                    }
                 }
                 UserEvent::ImageError(error) => {
                     self.error_visible = true;
@@ -86,17 +108,7 @@ impl App {
                             MouseScrollDelta::PixelDelta(pos) => pos.y as f32,
                         };
 
-                        let old_scale = image.scale;
-                        image.scale += image.scale * scroll as f32 / 10.0;
-
-                        let new_size = image.scaled();
-                        if new_size.x() < 100.0 || new_size.y() < 100.0 {
-                            image.scale = old_scale;
-                        } else {
-                            let mouse_to_center = image.position - self.mouse_position;
-                            image.position -=
-                                mouse_to_center * (old_scale - image.scale) / old_scale;
-                        }
+                        zoom(image, scroll, self.mouse_position);
                     }
                 }
                 WindowEvent::ModifiersChanged(state) => self.modifiers = *state,
@@ -123,15 +135,27 @@ impl App {
                                 VirtualKeyCode::S => {
                                     move_image(&mut self.image_view, Vec2::new(0.0, 20.0))
                                 }
-                                
+
                                 VirtualKeyCode::Q => rotate_image(&mut self.image_view, 90.0),
                                 VirtualKeyCode::E => rotate_image(&mut self.image_view, -90.0),
-                                
+
                                 VirtualKeyCode::R => {
                                     if let Some(image) = self.image_view.as_ref() {
                                         load_image(self.proxy.clone(), &image.path);
                                     }
-                                },
+                                }
+
+                                VirtualKeyCode::Left => {
+                                    if let Some(path) = self.image_list.previous() {
+                                        load_image(self.proxy.clone(), path);
+                                    }
+                                }
+
+                                VirtualKeyCode::Right => {
+                                    if let Some(path) = self.image_list.next() {
+                                        load_image(self.proxy.clone(), path);
+                                    }
+                                }
 
                                 VirtualKeyCode::F11 => {
                                     let window_context = display.gl_window();
@@ -157,6 +181,19 @@ impl App {
                         }
                     }
                 }
+                WindowEvent::ReceivedCharacter(c) => match c {
+                    '+' => {
+                        if let Some(image) = self.image_view.as_mut() {
+                            zoom(image, 1.0, self.size / 2.0)
+                        }
+                    }
+                    '-' => {
+                        if let Some(image) = self.image_view.as_mut() {
+                            zoom(image, -1.0, self.size / 2.0)
+                        }
+                    }
+                    _ => (),
+                },
                 _ => (),
             };
         }
@@ -282,6 +319,22 @@ impl App {
 
                     ui.separator();
 
+                    if MenuItem::new(im_str!("Zoom in"))
+                        .shortcut(im_str!("+"))
+                        .build(&ui)
+                    {
+                        zoom(image, 1.0, self.size / 2.0);
+                    }
+
+                    if MenuItem::new(im_str!("Zoom out"))
+                        .shortcut(im_str!("-"))
+                        .build(&ui)
+                    {
+                        zoom(image, -1.0, image.size / 2.0);
+                    }
+
+                    ui.separator();
+
                     if MenuItem::new(im_str!("Flip Horizontal")).build(&ui) {
                         image.flip_horizontal(display);
                     }
@@ -332,8 +385,23 @@ impl App {
             .always_use_window_padding(true)
             .build(ui, || {
                 if let Some(image) = self.image_view.as_mut() {
+                    match self.arrows.build(&ui) {
+                        Action::Left => {
+                            if let Some(path) = self.image_list.previous() {
+                                load_image(self.proxy.clone(), path);
+                            }
+                        }
+                        Action::Right => {
+                            if let Some(path) = self.image_list.next() {
+                                load_image(self.proxy.clone(), path);
+                            }
+                        }
+                        Action::None => (),
+                    }
+
                     ui.same_line_with_spacing(0.0, 10.0);
                     ui.text(&self.current_filename);
+
                     ui.same_line_with_spacing(0.0, 20.0);
                     ui.text(&format!("{} x {}", image.size.x(), image.size.y()));
                     ui.same_line_with_spacing(0.0, 20.0);
@@ -383,13 +451,13 @@ impl App {
             let mut exit = false;
             let message = self.error_message.clone();
             Window::new(im_str!("Error"))
-                .size([250.0, 100.0], Condition::Always)
+                .size([350.0, 100.0], Condition::Always)
                 .position_pivot([0.5, 0.5])
                 .position(
                     [self.size.x() / 2.0, self.size.y() / 2.0],
                     Condition::Appearing,
                 )
-                .resizable(false)
+                .resizable(true)
                 .focus_on_appearing(false)
                 .always_use_window_padding(true)
                 .focused(true)
@@ -424,9 +492,7 @@ impl App {
                     ui.text(env!("CARGO_PKG_NAME"));
                     ui.text(env!("CARGO_PKG_DESCRIPTION"));
                     ui.text(env!("CARGO_PKG_VERSION"));
-                    if let Some(string) = option_env!("GIT_HASH") {
-                        ui.text(&format!("Commit: {}", string));
-                    }
+                    ui.text(&format!("Commit: {}", env!("GIT_HASH")));
                     if ui.button(im_str!("Ok"), [50.0, 30.0]) {
                         exit = true;
                     }
@@ -452,6 +518,8 @@ impl App {
             mouse_position: Vec2::default(),
             current_filename: String::new(),
             about_visible: false,
+            image_list: ImageList::new(),
+            arrows: Arrows::new(),
         }
     }
 }
@@ -487,15 +555,29 @@ fn open_load_image(proxy: EventLoopProxy<UserEvent>) {
     });
 }
 
+fn zoom(image: &mut ImageView, zoom: f32, mouse_position: Vec2<f32>) {
+    let old_scale = image.scale;
+    image.scale += image.scale * zoom as f32 / 10.0;
+
+    let new_size = image.scaled();
+    if new_size.x() < 100.0 || new_size.y() < 100.0 {
+        image.scale = old_scale;
+    } else {
+        let mouse_to_center = image.position - mouse_position;
+        image.position -= mouse_to_center * (old_scale - image.scale) / old_scale;
+    }
+}
+
 pub fn load_image(proxy: EventLoopProxy<UserEvent>, path: impl AsRef<Path>) {
     let path_buf = path.as_ref().to_path_buf();
     thread::spawn(move || {
+        let start = Instant::now();
         let file = fs::read(&path_buf);
         let bytes = match file {
             Ok(bytes) => bytes,
             Err(_) => {
                 let _ =
-                    proxy.send_event(UserEvent::ImageError(String::from("Unable to read file")));
+                    proxy.send_event(UserEvent::ImageError(format!("Error could not read: {}", path_buf.to_str().unwrap())));
                 return;
             }
         };
@@ -503,7 +585,7 @@ pub fn load_image(proxy: EventLoopProxy<UserEvent>, path: impl AsRef<Path>) {
         let format = match image::guess_format(&bytes) {
             Ok(format) => format,
             Err(_) => {
-                let _ = proxy.send_event(UserEvent::ImageError(String::from("Unknown format")));
+                let _ = proxy.send_event(UserEvent::ImageError(format!("Error unknown format: {}", path_buf.to_str().unwrap())));
                 return;
             }
         };
@@ -511,14 +593,12 @@ pub fn load_image(proxy: EventLoopProxy<UserEvent>, path: impl AsRef<Path>) {
         let image = match ImageReader::with_format(Cursor::new(&bytes), format).decode() {
             Ok(image) => image.into_rgba16(),
             Err(_) => {
-                let _ = proxy.send_event(UserEvent::ImageError(String::from(
-                    "Unable to decode image",
-                )));
+                let _ = proxy.send_event(UserEvent::ImageError(format!("Error decode image: {}", path_buf.to_str().unwrap())));
                 return;
             }
         };
 
-        let _ = proxy.send_event(UserEvent::ImageLoaded(image, path_buf));
+        let _ = proxy.send_event(UserEvent::ImageLoaded(image, path_buf, start));
     });
 }
 
